import type { Attachment } from '../../../../decorators/attachment/Attachment';
import type { ProposeCredentialOptions, RequestCredentialOptions } from '../../CredentialsModuleOptions';
import type { DeleteCredentialOptions, ServiceAcceptCredentialOptions, ServiceAcceptProposalOptions, ServiceAcceptRequestOptions, ServiceOfferCredentialOptions, ServiceRequestCredentialOptions } from '../../protocol';
import type { V1CredentialPreview } from '../../protocol/v1/V1CredentialPreview';
import type { CredentialExchangeRecord } from '../../repository/CredentialExchangeRecord';
import type { FormatServiceCredentialAttachmentFormats, CredentialFormatSpec, HandlerAutoAcceptOptions, FormatServiceOfferAttachmentFormats, FormatServiceProposeAttachmentFormats } from '../models/CredentialFormatServiceOptions';
import { AgentConfig } from '../../../../agent/AgentConfig';
import { EventEmitter } from '../../../../agent/EventEmitter';
import { IndyHolderService, IndyIssuerService } from '../../../indy';
import { IndyLedgerService } from '../../../ledger';
import { V2CredentialPreview } from '../../protocol/v2/V2CredentialPreview';
import { CredentialRepository } from '../../repository/CredentialRepository';
import { CredentialFormatService } from '../CredentialFormatService';
export declare class IndyCredentialFormatService extends CredentialFormatService {
    private indyIssuerService;
    private indyLedgerService;
    private indyHolderService;
    protected credentialRepository: CredentialRepository;
    private logger;
    constructor(credentialRepository: CredentialRepository, eventEmitter: EventEmitter, indyIssuerService: IndyIssuerService, indyLedgerService: IndyLedgerService, indyHolderService: IndyHolderService, agentConfig: AgentConfig);
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param options The object containing all the options for the proposed credential
     * @param messageType the type of message which can be Indy, JsonLd etc eg "CRED_20_PROPOSAL"
     * @returns object containing associated attachment, formats and filtersAttach elements
     *
     */
    createProposal(options: ProposeCredentialOptions): FormatServiceProposeAttachmentFormats;
    processProposal(options: ServiceAcceptProposalOptions, credentialRecord: CredentialExchangeRecord): Promise<void>;
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param options The object containing all the options for the credential offer
     * @param messageType the type of message which can be Indy, JsonLd etc eg "CRED_20_OFFER"
     * @returns object containing associated attachment, formats and offersAttach elements
     *
     */
    createOffer(options: ServiceOfferCredentialOptions): Promise<FormatServiceOfferAttachmentFormats>;
    processOffer(attachment: Attachment, credentialRecord: CredentialExchangeRecord): Promise<void>;
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param requestOptions The object containing all the options for the credential request
     * @param credentialRecord the credential record containing the offer from which this request
     * is derived
     * @returns object containing associated attachment, formats and requestAttach elements
     *
     */
    createRequest(options: ServiceRequestCredentialOptions, credentialRecord: CredentialExchangeRecord, holderDid: string): Promise<FormatServiceCredentialAttachmentFormats>;
    /**
     * Not implemented; there for future versions
     */
    processRequest(_options: RequestCredentialOptions, _credentialRecord: CredentialExchangeRecord): Promise<void>;
    private getCredentialDefinition;
    /**
     * Get linked attachments for indy format from a proposal message. This allows attachments
     * to be copied across to old style credential records
     *
     * @param options ProposeCredentialOptions object containing (optionally) the linked attachments
     * @return array of linked attachments or undefined if none present
     */
    private getCredentialLinkedAttachments;
    /**
     * Gets the attachment object for a given attachId. We need to get out the correct attachId for
     * indy and then find the corresponding attachment (if there is one)
     * @param formats the formats object containing the attachid
     * @param messageAttachment the attachment containing the payload
     * @returns The Attachment if found or undefined
     */
    getAttachment(formats: CredentialFormatSpec[], messageAttachment: Attachment[]): Attachment | undefined;
    /**
     * Create a credential offer for the given credential definition id.
     *
     * @param credentialDefinitionId The credential definition to create an offer for
     * @returns The created credential offer
     */
    private createCredentialOffer;
    /**
     * Create a credential offer for the given credential definition id.
     *
     * @param options RequestCredentialOptions the config options for the credential request
     * @throws Error if unable to create the request
     * @returns The created credential offer
     */
    private createIndyCredentialRequest;
    /**
     * Create a {@link AttachmentFormats} object dependent on the message type.
     *
     * @param requestOptions The object containing all the options for the credential request
     * @param credentialRecord the credential record containing the offer from which this request
     * is derived
     * @returns object containing associated attachment, formats and requestAttach elements
     *
     */
    createCredential(options: ServiceAcceptRequestOptions, record: CredentialExchangeRecord, requestAttachment: Attachment, offerAttachment?: Attachment): Promise<FormatServiceCredentialAttachmentFormats>;
    /**
     * Processes an incoming credential - retrieve metadata, retrieve payload and store it in the Indy wallet
     * @param message the issue credential message
     */
    /**
     * Processes an incoming credential - retrieve metadata, retrieve payload and store it in the Indy wallet
     * @param options the issue credential message wrapped inside this object
     * @param credentialRecord the credential exchange record for this credential
     */
    processCredential(options: ServiceAcceptCredentialOptions, credentialRecord: CredentialExchangeRecord): Promise<void>;
    /**
     * Checks whether it should automatically respond to a proposal. Moved from CredentialResponseCoordinator
     * as this contains format-specific logic
     * @param credentialRecord The credential record for which we are testing whether or not to auto respond
     * @param agentConfig config object for the agent, used to hold auto accept state for the agent
     * @returns true if we should auto respond, false otherwise
     */
    shouldAutoRespondToProposal(handlerOptions: HandlerAutoAcceptOptions): boolean;
    /**
   * Checks whether it should automatically respond to a request. Moved from CredentialResponseCoordinator
   * as this contains format-specific logic
   * @param credentialRecord The credential record for which we are testing whether or not to auto respond
   * @param autoAcceptType auto accept type for this credential exchange - normal auto or content approved
   * @returns true if we should auto respond, false otherwise
  
   */
    shouldAutoRespondToRequest(options: HandlerAutoAcceptOptions): boolean;
    /**
     * Checks whether it should automatically respond to a request. Moved from CredentialResponseCoordinator
     * as this contains format-specific logic
     * @param credentialRecord The credential record for which we are testing whether or not to auto respond
     * @param autoAcceptType auto accept type for this credential exchange - normal auto or content approved
     * @returns true if we should auto respond, false otherwise
     */
    shouldAutoRespondToCredential(options: HandlerAutoAcceptOptions): boolean;
    private areProposalValuesValid;
    private areProposalAndOfferDefinitionIdEqual;
    private areCredentialValuesValid;
    deleteCredentialById(credentialRecord: CredentialExchangeRecord, options: DeleteCredentialOptions): Promise<void>;
    checkPreviewAttributesMatchSchemaAttributes(offerAttachment: Attachment, preview: V1CredentialPreview | V2CredentialPreview): Promise<void>;
    private isRequestDefinitionIdValid;
    private generateId;
    private getRevocationRegistry;
}
