import type { AgentMessage } from '../../../../agent/AgentMessage';
import type { HandlerInboundMessage } from '../../../../agent/Handler';
import type { InboundMessageContext } from '../../../../agent/models/InboundMessageContext';
import type { CredentialOfferTemplate, CredentialProtocolMsgReturnType, ServiceAcceptRequestOptions, ServiceRequestCredentialOptions } from '../../CredentialServiceOptions';
import type { AcceptProposalOptions, NegotiateProposalOptions, OfferCredentialOptions, ProposeCredentialOptions } from '../../CredentialsModuleOptions';
import type { CredentialFormatService } from '../../formats/CredentialFormatService';
import type { HandlerAutoAcceptOptions } from '../../formats/models/CredentialFormatServiceOptions';
import { AgentConfig } from '../../../../agent/AgentConfig';
import { Dispatcher } from '../../../../agent/Dispatcher';
import { EventEmitter } from '../../../../agent/EventEmitter';
import { DidCommMessageRepository } from '../../../../storage';
import { ConnectionService } from '../../../connections/services';
import { MediationRecipientService } from '../../../routing';
import { CredentialProtocolVersion } from '../../CredentialProtocolVersion';
import { IndyCredentialFormatService } from '../../formats/indy/IndyCredentialFormatService';
import { CredentialRepository, CredentialExchangeRecord } from '../../repository';
import { CredentialService, RevocationService } from '../../services';
import { V1OfferCredentialHandler } from './handlers';
import { V1ProposeCredentialMessage, V1IssueCredentialMessage, V1RequestCredentialMessage, V1OfferCredentialMessage, V1CredentialAckMessage } from './messages';
export declare class V1CredentialService extends CredentialService {
    private connectionService;
    private formatService;
    constructor(connectionService: ConnectionService, didCommMessageRepository: DidCommMessageRepository, agentConfig: AgentConfig, mediationRecipientService: MediationRecipientService, dispatcher: Dispatcher, eventEmitter: EventEmitter, credentialRepository: CredentialRepository, formatService: IndyCredentialFormatService, revocationService: RevocationService);
    /**
     * Create a {@link ProposeCredentialMessage} not bound to an existing credential exchange.
     * To create a proposal as response to an existing credential exchange, use {@link createProposalAsResponse}.
     *
     * @param proposal The object containing config options
     * @returns Object containing proposal message and associated credential record
     *
     */
    createProposal(proposal: ProposeCredentialOptions): Promise<CredentialProtocolMsgReturnType<V1ProposeCredentialMessage>>;
    /**
     * Processing an incoming credential message and create a credential offer as a response
     * @param proposal The object containing config options
     * @param credentialRecord the credential exchange record for this proposal
     * @returns Object containing proposal message and associated credential record
     */
    acceptProposal(options: AcceptProposalOptions, credentialRecord: CredentialExchangeRecord): Promise<CredentialProtocolMsgReturnType<V1OfferCredentialMessage>>;
    /**
     * Negotiate a credential proposal as issuer (by sending a credential offer message) to the connection
     * associated with the credential record.
     *
     * @param credentialOptions configuration for the offer see {@link NegotiateProposalOptions}
     * @param credentialRecord the credential exchange record for this proposal
     * @returns Credential record associated with the credential offer and the corresponding new offer message
     *
     */
    negotiateProposal(credentialOptions: NegotiateProposalOptions, credentialRecord: CredentialExchangeRecord): Promise<CredentialProtocolMsgReturnType<V1OfferCredentialMessage>>;
    /**
     * Process a received {@link ProposeCredentialMessage}. This will not accept the credential proposal
     * or send a credential offer. It will only create a new, or update the existing credential record with
     * the information from the credential proposal message. Use {@link createOfferAsResponse}
     * after calling this method to create a credential offer.
     *
     * @param messageContext The message context containing a credential proposal message
     * @returns credential record associated with the credential proposal message
     *
     */
    processProposal(messageContext: InboundMessageContext<V1ProposeCredentialMessage>): Promise<CredentialExchangeRecord>;
    /**
     * Create a {@link OfferCredentialMessage} as response to a received credential proposal.
     * To create an offer not bound to an existing credential exchange, use {@link createOffer}.
     *
     * @param credentialRecord The credential record for which to create the credential offer
     * @param credentialTemplate The credential template to use for the offer
     * @returns Object containing offer message and associated credential record
     *
     */
    createOfferAsResponse(credentialRecord: CredentialExchangeRecord, credentialTemplate: CredentialOfferTemplate): Promise<CredentialProtocolMsgReturnType<V1OfferCredentialMessage>>;
    /**
     * Process a received {@link RequestCredentialMessage}. This will not accept the credential request
     * or send a credential. It will only update the existing credential record with
     * the information from the credential request message. Use {@link createCredential}
     * after calling this method to create a credential.
     *
     * @param messageContext The message context containing a credential request message
     * @returns credential record associated with the credential request message
     *
     */
    negotiateOffer(credentialOptions: ProposeCredentialOptions, credentialRecord: CredentialExchangeRecord): Promise<CredentialProtocolMsgReturnType<V1ProposeCredentialMessage>>;
    /**
     * Create a {@link OfferCredentialMessage} not bound to an existing credential exchange.
     * To create an offer as response to an existing credential exchange, use {@link V1CredentialService#createOfferAsResponse}.
     *
     * @param credentialOptions The options containing config params for creating the credential offer
     * @returns Object containing offer message and associated credential record
     *
     */
    createOffer(credentialOptions: OfferCredentialOptions): Promise<CredentialProtocolMsgReturnType<V1OfferCredentialMessage>>;
    /**
     * Process a received {@link OfferCredentialMessage}. This will not accept the credential offer
     * or send a credential request. It will only create a new credential record with
     * the information from the credential offer message. Use {@link createRequest}
     * after calling this method to create a credential request.
     *
     * @param messageContext The message context containing a credential request message
     * @returns credential record associated with the credential offer message
     *
     */
    processOffer(messageContext: HandlerInboundMessage<V1OfferCredentialHandler>): Promise<CredentialExchangeRecord>;
    private createOfferProcessing;
    createOutOfBandOffer(credentialOptions: OfferCredentialOptions): Promise<CredentialProtocolMsgReturnType<V1OfferCredentialMessage>>;
    /**
     * Create a {@link RequestCredentialMessage} as response to a received credential offer.
     *
     * @param record The credential record for which to create the credential request
     * @param options Additional configuration to use for the credential request
     * @returns Object containing request message and associated credential record
     *
     */
    createRequest(record: CredentialExchangeRecord, options: ServiceRequestCredentialOptions, holderDid: string): Promise<CredentialProtocolMsgReturnType<V1RequestCredentialMessage>>;
    /**
     * Process a received {@link IssueCredentialMessage}. This will not accept the credential
     * or send a credential acknowledgement. It will only update the existing credential record with
     * the information from the issue credential message. Use {@link createAck}
     * after calling this method to create a credential acknowledgement.
     *
     * @param messageContext The message context containing an issue credential message
     *
     * @returns credential record associated with the issue credential message
     *
     */
    processRequest(messageContext: InboundMessageContext<V1RequestCredentialMessage>): Promise<CredentialExchangeRecord>;
    /**
     * Create a {@link IssueCredentialMessage} as response to a received credential request.
     *
     * @param record The credential record for which to create the credential
     * @param options Additional configuration to use for the credential
     * @returns Object containing issue credential message and associated credential record
     *
     */
    createCredential(record: CredentialExchangeRecord, options: ServiceAcceptRequestOptions): Promise<CredentialProtocolMsgReturnType<V1IssueCredentialMessage>>;
    /**
     * Process an incoming {@link IssueCredentialMessage}
     *
     * @param messageContext The message context containing a credential acknowledgement message
     * @returns credential record associated with the credential acknowledgement message
     *
     */
    processCredential(messageContext: InboundMessageContext<V1IssueCredentialMessage>): Promise<CredentialExchangeRecord>;
    /**
     * Process a received {@link CredentialAckMessage}.
     *
     * @param messageContext The message context containing a credential acknowledgement message
     * @returns credential record associated with the credential acknowledgement message
     *
     */
    /**
     * Create a {@link CredentialAckMessage} as response to a received credential.
     *
     * @param credentialRecord The credential record for which to create the credential acknowledgement
     * @returns Object containing credential acknowledgement message and associated credential record
     *
     */
    createAck(credentialRecord: CredentialExchangeRecord): Promise<CredentialProtocolMsgReturnType<V1CredentialAckMessage>>;
    processAck(messageContext: InboundMessageContext<V1CredentialAckMessage>): Promise<CredentialExchangeRecord>;
    registerHandlers(): void;
    /**
     *
     * Get the version of Issue Credentials according to AIP1.0 or AIP2.0
     * @returns the version of this credential service
     */
    getVersion(): CredentialProtocolVersion;
    /**
     * Negotiate a credential offer as holder (by sending a credential proposal message) to the connection
     * associated with the credential record.
     *
     * @param credentialOptions configuration for the offer see {@link NegotiateProposalOptions}
     * @param credentialRecord the credential exchange record for this proposal
     * @returns Credential record associated with the credential offer and the corresponding new offer message
     *
     */
    shouldAutoRespondToCredential(credentialRecord: CredentialExchangeRecord, credentialMessage: V1IssueCredentialMessage): boolean;
    shouldAutoRespondToProposal(handlerOptions: HandlerAutoAcceptOptions): Promise<boolean>;
    private areProposalValuesValid;
    private areProposalAndOfferDefinitionIdEqual;
    shouldAutoRespondToRequest(credentialRecord: CredentialExchangeRecord, requestMessage: V1RequestCredentialMessage, proposeMessage?: V1ProposeCredentialMessage, offerMessage?: V1OfferCredentialMessage): boolean;
    shouldAutoRespondToOffer(credentialRecord: CredentialExchangeRecord, offerMessage: V1OfferCredentialMessage, proposeMessage?: V1ProposeCredentialMessage): boolean;
    getOfferMessage(id: string): Promise<AgentMessage | null>;
    getRequestMessage(id: string): Promise<AgentMessage | null>;
    getCredentialMessage(id: string): Promise<AgentMessage | null>;
    getFormats(): CredentialFormatService[];
    getFormatService(): CredentialFormatService;
}
