"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.V1OfferCredentialHandler = void 0;
const helpers_1 = require("../../../../../agent/helpers");
const ServiceDecorator_1 = require("../../../../../decorators/service/ServiceDecorator");
const AriesFrameworkError_1 = require("../../../../../error/AriesFrameworkError");
const storage_1 = require("../../../../../storage");
const messages_1 = require("../messages");
class V1OfferCredentialHandler {
    constructor(credentialService, agentConfig, mediationRecipientService, didCommMessageRepository) {
        this.supportedMessages = [messages_1.V1OfferCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.mediationRecipientService = mediationRecipientService;
        this.didCommMessageRepository = didCommMessageRepository;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processOffer(messageContext);
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        if (!offerMessage) {
            throw new AriesFrameworkError_1.AriesFrameworkError('Missing offerMessage in V1OfferCredentialHandler');
        }
        const proposeMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1ProposeCredentialMessage,
        });
        const shouldAutoRespond = this.credentialService.shouldAutoRespondToOffer(credentialRecord, offerMessage, proposeMessage !== null && proposeMessage !== void 0 ? proposeMessage : undefined);
        if (shouldAutoRespond) {
            return await this.createRequest(credentialRecord, messageContext, offerMessage);
        }
    }
    async createRequest(record, messageContext, offerMessage) {
        this.agentConfig.logger.info(`Automatically sending request with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        if (messageContext.connection) {
            const { message, credentialRecord } = await this.credentialService.createRequest(record, {}, messageContext.connection.did);
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: message,
                role: storage_1.DidCommMessageRole.Sender,
                associatedRecordId: credentialRecord.id,
            });
            return helpers_1.createOutboundMessage(messageContext.connection, message);
        }
        else if (offerMessage === null || offerMessage === void 0 ? void 0 : offerMessage.service) {
            const routing = await this.mediationRecipientService.getRouting();
            const ourService = new ServiceDecorator_1.ServiceDecorator({
                serviceEndpoint: routing.endpoints[0],
                recipientKeys: [routing.verkey],
                routingKeys: routing.routingKeys,
            });
            const recipientService = offerMessage.service;
            const { message, credentialRecord } = await this.credentialService.createRequest(record, {}, ourService.recipientKeys[0]);
            // Set and save ~service decorator to record (to remember our verkey)
            message.service = ourService;
            await this.credentialService.update(credentialRecord);
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: message,
                role: storage_1.DidCommMessageRole.Sender,
                associatedRecordId: credentialRecord.id,
            });
            return helpers_1.createOutboundServiceMessage({
                payload: message,
                service: recipientService.toDidCommService(),
                senderKey: ourService.recipientKeys[0],
            });
        }
        this.agentConfig.logger.error(`Could not automatically create credential request`);
    }
}
exports.V1OfferCredentialHandler = V1OfferCredentialHandler;
//# sourceMappingURL=V1OfferCredentialHandler.js.map