"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.V1ProposeCredentialHandler = void 0;
const helpers_1 = require("../../../../../agent/helpers");
const AriesFrameworkError_1 = require("../../../../../error/AriesFrameworkError");
const CredentialAutoAcceptType_1 = require("../../../CredentialAutoAcceptType");
const messages_1 = require("../messages");
class V1ProposeCredentialHandler {
    constructor(credentialService, agentConfig, didCommMessageRepository) {
        this.supportedMessages = [messages_1.V1ProposeCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.didCommMessageRepository = didCommMessageRepository;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processProposal(messageContext);
        // note that these two messages can be present (or not) and there is no
        // guarantee which one is present so we need two try-catch blocks
        const proposalMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1ProposeCredentialMessage,
        });
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        let proposalValues;
        if (!proposalMessage || !proposalMessage.credentialProposal || !proposalMessage.credentialProposal.attributes) {
            throw new AriesFrameworkError_1.AriesFrameworkError('Missing attributes in proposal message');
        }
        let proposalAttachment, offerAttachment;
        if (proposalMessage) {
            proposalValues = proposalMessage.credentialProposal.attributes;
        }
        if (offerMessage) {
            offerAttachment = offerMessage.getAttachmentById('indy');
        }
        const handlerOptions = {
            credentialRecord,
            autoAcceptType: this.agentConfig.autoAcceptCredentials,
            messageAttributes: proposalValues,
            proposalAttachment,
            offerAttachment,
            credentialDefinitionId: proposalMessage.credentialDefinitionId,
        };
        if (this.agentConfig.autoAcceptCredentials === CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
            credentialRecord.autoAcceptCredential === CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
            (await this.credentialService.shouldAutoRespondToProposal(handlerOptions))) {
            return await this.createOffer(credentialRecord, messageContext, proposalMessage);
        }
    }
    async createOffer(credentialRecord, messageContext, proposalMessage) {
        this.agentConfig.logger.info(`Automatically sending offer with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        if (!messageContext.connection) {
            this.agentConfig.logger.error('No connection on the messageContext, aborting auto accept');
            return;
        }
        if (!(proposalMessage === null || proposalMessage === void 0 ? void 0 : proposalMessage.credentialProposal)) {
            this.agentConfig.logger.error(`Proposal message with id ${credentialRecord.id} is missing required credential proposal`);
            return;
        }
        if (!proposalMessage.credentialDefinitionId) {
            this.agentConfig.logger.error('Missing required credential definition id');
            return;
        }
        const { message } = await this.credentialService.createOfferAsResponse(credentialRecord, {
            credentialDefinitionId: proposalMessage.credentialDefinitionId,
            preview: proposalMessage.credentialProposal,
        });
        return helpers_1.createOutboundMessage(messageContext.connection, message);
    }
}
exports.V1ProposeCredentialHandler = V1ProposeCredentialHandler;
//# sourceMappingURL=V1ProposeCredentialHandler.js.map