"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.V1RequestCredentialHandler = void 0;
const helpers_1 = require("../../../../../agent/helpers");
const storage_1 = require("../../../../../storage");
const CredentialAutoAcceptType_1 = require("../../../CredentialAutoAcceptType");
const messages_1 = require("../messages");
class V1RequestCredentialHandler {
    constructor(credentialService, agentConfig, didCommMessageRepository) {
        this.supportedMessages = [messages_1.V1RequestCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.didCommMessageRepository = didCommMessageRepository;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processRequest(messageContext);
        const requestMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1RequestCredentialMessage,
        });
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        const proposeMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1ProposeCredentialMessage,
        });
        const formatService = this.credentialService.getFormatService();
        let proposalAttachment, offerAttachment, requestAttachment;
        if (proposeMessage && proposeMessage.appendedAttachments) {
            proposalAttachment = proposeMessage.appendedAttachments[0];
        }
        if (offerMessage) {
            offerAttachment = offerMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID);
        }
        if (requestMessage) {
            requestAttachment = requestMessage.getAttachmentById(messages_1.INDY_CREDENTIAL_REQUEST_ATTACHMENT_ID);
        }
        const handlerOptions = {
            credentialRecord,
            autoAcceptType: this.agentConfig.autoAcceptCredentials,
            proposalAttachment,
            offerAttachment,
            requestAttachment,
        };
        if (this.agentConfig.autoAcceptCredentials === CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
            credentialRecord.autoAcceptCredential === CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
            formatService.shouldAutoRespondToRequest(handlerOptions)) {
            return await this.createCredential(credentialRecord, messageContext, offerMessage, requestMessage);
        }
    }
    async createCredential(record, messageContext, offerMessage, requestMessage) {
        this.agentConfig.logger.info(`Automatically sending credential with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        const options = {
            attachId: messages_1.INDY_CREDENTIAL_ATTACHMENT_ID,
            credentialRecordId: record.id,
            comment: 'V1 Indy Credential',
        };
        const { message, credentialRecord } = await this.credentialService.createCredential(record, options);
        if (messageContext.connection) {
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: message,
                role: storage_1.DidCommMessageRole.Sender,
                associatedRecordId: credentialRecord.id,
            });
            return helpers_1.createOutboundMessage(messageContext.connection, message);
        }
        else if ((requestMessage === null || requestMessage === void 0 ? void 0 : requestMessage.service) && (offerMessage === null || offerMessage === void 0 ? void 0 : offerMessage.service)) {
            const recipientService = requestMessage.service;
            const ourService = offerMessage.service;
            // Set ~service, update message in record (for later use)
            message.setService(ourService);
            await this.credentialService.update(credentialRecord);
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: message,
                role: storage_1.DidCommMessageRole.Sender,
                associatedRecordId: credentialRecord.id,
            });
            return helpers_1.createOutboundServiceMessage({
                payload: message,
                service: recipientService.toDidCommService(),
                senderKey: ourService.recipientKeys[0],
            });
        }
        this.agentConfig.logger.error(`Could not automatically create credential request`);
    }
}
exports.V1RequestCredentialHandler = V1RequestCredentialHandler;
//# sourceMappingURL=V1RequestCredentialHandler.js.map