"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CredentialMessageBuilder = void 0;
const AriesFrameworkError_1 = require("../../../../error/AriesFrameworkError");
const uuid_1 = require("../../../../utils/uuid");
const CredentialProtocolVersion_1 = require("../../CredentialProtocolVersion");
const CredentialState_1 = require("../../CredentialState");
const CredentialExchangeRecord_1 = require("../../repository/CredentialExchangeRecord");
const V2CredentialPreview_1 = require("./V2CredentialPreview");
const V2IssueCredentialMessage_1 = require("./messages/V2IssueCredentialMessage");
const V2OfferCredentialMessage_1 = require("./messages/V2OfferCredentialMessage");
const V2ProposeCredentialMessage_1 = require("./messages/V2ProposeCredentialMessage");
const V2RequestCredentialMessage_1 = require("./messages/V2RequestCredentialMessage");
class CredentialMessageBuilder {
    /**
     * Create a v2 credential proposal message according to the logic contained in the format service. The format services
     * contain specific logic related to indy, jsonld etc. with others to come.
     *
     * @param formats {@link CredentialFormatService} array of format service objects each containing format-specific logic
     * @param proposal {@link ProposeCredentialOptions} object containing (optionally) the linked attachments
     * @param _threadId optional thread id for this message service
     * @return a version 2.0 credential propose message see {@link V2ProposeCredentialMessage}
     */
    createProposal(formatServices, proposal) {
        if (formatServices.length === 0) {
            throw new AriesFrameworkError_1.AriesFrameworkError('no format services provided to createProposal');
        }
        // create message
        // there are two arrays in each message, one for formats the other for attachments
        const formatsArray = [];
        const filtersAttachArray = [];
        let previewAttachments;
        for (const formatService of formatServices) {
            const { format: formats, attachment, preview } = formatService.createProposal(proposal);
            if (attachment) {
                filtersAttachArray.push(attachment);
            }
            else {
                throw new AriesFrameworkError_1.AriesFrameworkError('attachment not initialized for credential proposal');
            }
            if (preview) {
                previewAttachments = preview;
            }
            formatsArray.push(formats);
        }
        const options = {
            id: this.generateId(),
            formats: formatsArray,
            filtersAttach: filtersAttachArray,
            comment: proposal.comment,
            credentialProposal: previewAttachments,
        };
        const message = new V2ProposeCredentialMessage_1.V2ProposeCredentialMessage(options);
        const props = {
            connectionId: proposal.connectionId,
            threadId: message.threadId,
            state: CredentialState_1.CredentialState.ProposalSent,
            autoAcceptCredential: proposal === null || proposal === void 0 ? void 0 : proposal.autoAcceptCredential,
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V2,
            credentials: [],
        };
        // Create the v2 record
        const credentialRecord = new CredentialExchangeRecord_1.CredentialExchangeRecord(props);
        return { message, credentialRecord };
    }
    /**
     * accept a v2 credential proposal message according to the logic contained in the format service. The format services
     * contain specific logic related to indy, jsonld etc. with others to come.
     *
     * @param message {@link V2ProposeCredentialMessage} object containing (optionally) the linked attachments
     * @param connectionId optional connection id for the agent to agent connection
     * @return a version 2.0 credential record object see {@link CredentialRecord}
     */
    processProposal(message, connectionId) {
        var _a;
        const props = {
            connectionId: connectionId,
            threadId: message.threadId,
            state: CredentialState_1.CredentialState.ProposalReceived,
            credentialAttributes: (_a = message.credentialProposal) === null || _a === void 0 ? void 0 : _a.attributes,
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V2,
            credentials: [],
        };
        return new CredentialExchangeRecord_1.CredentialExchangeRecord(props);
    }
    async createOfferAsResponse(formatServices, credentialRecord, options) {
        if (formatServices.length === 0) {
            throw new AriesFrameworkError_1.AriesFrameworkError('no format services provided to createProposal');
        }
        // create message
        // there are two arrays in each message, one for formats the other for attachments
        const formatsArray = [];
        const offersAttachArray = [];
        let previewAttachments = new V2CredentialPreview_1.V2CredentialPreview({
            attributes: [],
        });
        for (const formatService of formatServices) {
            const { attachment: offersAttach, preview, format } = await formatService.createOffer(options);
            if (offersAttach === undefined) {
                throw new AriesFrameworkError_1.AriesFrameworkError('offersAttach not initialized for credential offer');
            }
            if (offersAttach) {
                offersAttachArray.push(offersAttach);
            }
            else {
                throw new AriesFrameworkError_1.AriesFrameworkError('offersAttach not initialized for credential proposal');
            }
            if (preview && preview.attributes.length > 0) {
                previewAttachments = preview;
            }
            formatsArray.push(format);
            await formatService.processOffer(offersAttach, credentialRecord);
        }
        const messageProps = {
            id: this.generateId(),
            formats: formatsArray,
            comment: options.comment,
            offerAttachments: offersAttachArray,
            credentialPreview: previewAttachments,
        };
        const credentialOfferMessage = new V2OfferCredentialMessage_1.V2OfferCredentialMessage(messageProps);
        credentialOfferMessage.setThread({
            threadId: credentialRecord.threadId,
        });
        credentialRecord.credentialAttributes = previewAttachments === null || previewAttachments === void 0 ? void 0 : previewAttachments.attributes;
        return credentialOfferMessage;
    }
    /**
     * Create a {@link V2RequestCredentialMessage}
     *
     * @param formatService correct service for format, indy, w3c etc.
     * @param record The credential record for which to create the credential request
     * @param offer Additional configuration for the offer if present (might not be for W3C)
     * @returns Object containing request message and associated credential record
     *
     */
    async createRequest(options) {
        var _a;
        if (options.formatServices.length === 0) {
            throw new AriesFrameworkError_1.AriesFrameworkError('no format services provided to createProposal');
        }
        const formatsArray = [];
        const requestAttachArray = [];
        for (const format of options.formatServices) {
            // use the attach id in the formats object to find the correct attachment
            const attachment = format.getAttachment(options.offerMessage.formats, options.offerMessage.messageAttachment);
            if (attachment) {
                options.requestOptions.offerAttachment = attachment;
            }
            else {
                throw new AriesFrameworkError_1.AriesFrameworkError(`Missing data payload in attachment in credential Record ${options.record.id}`);
            }
            const { format: formats, attachment: requestAttach } = await format.createRequest(options.requestOptions, options.record, options.holderDid);
            options.requestOptions.requestAttachment = requestAttach;
            if (formats && requestAttach) {
                formatsArray.push(formats);
                requestAttachArray.push(requestAttach);
            }
        }
        const messageOptions = {
            id: this.generateId(),
            formats: formatsArray,
            requestsAttach: requestAttachArray,
            comment: options.requestOptions.comment,
        };
        const credentialRequestMessage = new V2RequestCredentialMessage_1.V2RequestCredentialMessage(messageOptions);
        credentialRequestMessage.setThread({ threadId: options.record.threadId });
        options.record.autoAcceptCredential =
            (_a = options.requestOptions.autoAcceptCredential) !== null && _a !== void 0 ? _a : options.record.autoAcceptCredential;
        return { message: credentialRequestMessage, credentialRecord: options.record };
    }
    /**
     * Create a {@link V2OfferCredentialMessage} as beginning of protocol process.
     *
     * @param formatService {@link CredentialFormatService} the format service object containing format-specific logic
     * @param options attributes of the original offer
     * @returns Object containing offer message and associated credential record
     *
     */
    async createOffer(formatServices, options) {
        if (formatServices.length === 0) {
            throw new AriesFrameworkError_1.AriesFrameworkError('no format services provided to createProposal');
        }
        const formatsArray = [];
        const offersAttachArray = [];
        let previewAttachments = new V2CredentialPreview_1.V2CredentialPreview({
            attributes: [],
        });
        const offerMap = new Map();
        for (const formatService of formatServices) {
            const { attachment: offersAttach, preview, format } = await formatService.createOffer(options);
            if (offersAttach) {
                offersAttachArray.push(offersAttach);
                offerMap.set(offersAttach, formatService);
            }
            else {
                throw new AriesFrameworkError_1.AriesFrameworkError('offersAttach not initialized for credential proposal');
            }
            if (preview) {
                previewAttachments = preview;
            }
            formatsArray.push(format);
        }
        const messageProps = {
            id: this.generateId(),
            formats: formatsArray,
            comment: options.comment,
            offerAttachments: offersAttachArray,
            replacementId: undefined,
            credentialPreview: previewAttachments,
        };
        // Construct v2 offer message
        const credentialOfferMessage = new V2OfferCredentialMessage_1.V2OfferCredentialMessage(messageProps);
        const recordProps = {
            connectionId: options.connectionId,
            threadId: credentialOfferMessage.threadId,
            autoAcceptCredential: options === null || options === void 0 ? void 0 : options.autoAcceptCredential,
            state: CredentialState_1.CredentialState.OfferSent,
            credentialAttributes: previewAttachments === null || previewAttachments === void 0 ? void 0 : previewAttachments.attributes,
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V2,
            credentials: [],
        };
        const credentialRecord = new CredentialExchangeRecord_1.CredentialExchangeRecord(recordProps);
        for (const offersAttach of offerMap.keys()) {
            const service = offerMap.get(offersAttach);
            if (!service) {
                throw new AriesFrameworkError_1.AriesFrameworkError(`No service found for attachment: ${offersAttach.id}`);
            }
            await service.processOffer(offersAttach, credentialRecord);
        }
        return { credentialRecord, message: credentialOfferMessage };
    }
    /**
     * Create a {@link V2IssueCredentialMessage} - we issue the credentials to the holder with this message
     *
     * @param formatService {@link CredentialFormatService} the format service object containing format-specific logic
     * @param offerMessage the original offer message
     * @returns Object containing offer message and associated credential record
     *
     */
    async createCredential(credentialFormats, record, serviceOptions, requestMessage, offerMessage) {
        const formatsArray = [];
        const credAttachArray = [];
        for (const formatService of credentialFormats) {
            const offerAttachment = formatService.getAttachment(offerMessage.formats, offerMessage.messageAttachment);
            const requestAttachment = formatService.getAttachment(requestMessage.formats, requestMessage.messageAttachment);
            if (!requestAttachment) {
                throw new Error(`Missing request attachment in createCredential`);
            }
            const { format: formats, attachment: credentialsAttach } = await formatService.createCredential(serviceOptions, record, requestAttachment, offerAttachment);
            if (!formats) {
                throw new AriesFrameworkError_1.AriesFrameworkError('formats not initialized for credential');
            }
            formatsArray.push(formats);
            if (!credentialsAttach) {
                throw new AriesFrameworkError_1.AriesFrameworkError('credentialsAttach not initialized for credential');
            }
            credAttachArray.push(credentialsAttach);
        }
        const messageOptions = {
            id: this.generateId(),
            formats: formatsArray,
            credentialsAttach: credAttachArray,
            comment: serviceOptions.comment,
        };
        const message = new V2IssueCredentialMessage_1.V2IssueCredentialMessage(messageOptions);
        return { message, credentialRecord: record };
    }
    generateId() {
        return uuid_1.uuid();
    }
}
exports.CredentialMessageBuilder = CredentialMessageBuilder;
//# sourceMappingURL=CredentialMessageBuilder.js.map