"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.V2CredentialService = void 0;
const tsyringe_1 = require("tsyringe");
const AgentConfig_1 = require("../../../../agent/AgentConfig");
const Dispatcher_1 = require("../../../../agent/Dispatcher");
const EventEmitter_1 = require("../../../../agent/EventEmitter");
const ServiceDecorator_1 = require("../../../../decorators/service/ServiceDecorator");
const error_1 = require("../../../../error");
const storage_1 = require("../../../../storage");
const common_1 = require("../../../common");
const ConnectionService_1 = require("../../../connections/services/ConnectionService");
const routing_1 = require("../../../routing");
const CredentialAutoAcceptType_1 = require("../../CredentialAutoAcceptType");
const CredentialEvents_1 = require("../../CredentialEvents");
const CredentialProtocolVersion_1 = require("../../CredentialProtocolVersion");
const CredentialState_1 = require("../../CredentialState");
const CredentialsModuleOptions_1 = require("../../CredentialsModuleOptions");
const errors_1 = require("../../errors");
const IndyCredentialFormatService_1 = require("../../formats/indy/IndyCredentialFormatService");
const CredentialFormatServiceOptions_1 = require("../../formats/models/CredentialFormatServiceOptions");
const repository_1 = require("../../repository");
const services_1 = require("../../services");
const CredentialService_1 = require("../../services/CredentialService");
const CredentialMessageBuilder_1 = require("./CredentialMessageBuilder");
const V2CredentialAckHandler_1 = require("./handlers/V2CredentialAckHandler");
const V2CredentialProblemReportHandler_1 = require("./handlers/V2CredentialProblemReportHandler");
const V2IssueCredentialHandler_1 = require("./handlers/V2IssueCredentialHandler");
const V2OfferCredentialHandler_1 = require("./handlers/V2OfferCredentialHandler");
const V2ProposeCredentialHandler_1 = require("./handlers/V2ProposeCredentialHandler");
const V2RequestCredentialHandler_1 = require("./handlers/V2RequestCredentialHandler");
const V2CredentialAckMessage_1 = require("./messages/V2CredentialAckMessage");
const V2IssueCredentialMessage_1 = require("./messages/V2IssueCredentialMessage");
const V2OfferCredentialMessage_1 = require("./messages/V2OfferCredentialMessage");
const V2ProposeCredentialMessage_1 = require("./messages/V2ProposeCredentialMessage");
const V2RequestCredentialMessage_1 = require("./messages/V2RequestCredentialMessage");
let V2CredentialService = class V2CredentialService extends CredentialService_1.CredentialService {
    constructor(connectionService, credentialRepository, eventEmitter, dispatcher, agentConfig, mediationRecipientService, didCommMessageRepository, indyCredentialFormatService, revocationService) {
        super(credentialRepository, eventEmitter, dispatcher, agentConfig, mediationRecipientService, didCommMessageRepository, revocationService);
        this.connectionService = connectionService;
        this.indyCredentialFormatService = indyCredentialFormatService;
        this.credentialMessageBuilder = new CredentialMessageBuilder_1.CredentialMessageBuilder();
        this.serviceFormatMap = {
            [CredentialsModuleOptions_1.CredentialFormatType.Indy]: this.indyCredentialFormatService,
        };
    }
    /**
     * Create a {@link V2ProposeCredentialMessage} not bound to an existing credential exchange.
     *
     * @param proposal The ProposeCredentialOptions object containing the important fields for the credential message
     * @returns Object containing proposal message and associated credential record
     *
     */
    async createProposal(proposal) {
        var _a;
        this.logger.debug('Get the Format Service and Create Proposal Message');
        const formats = this.getFormats(proposal.credentialFormats);
        if (!formats || formats.length === 0) {
            throw new error_1.AriesFrameworkError(`Unable to create proposal. No supported formats`);
        }
        const { message: proposalMessage, credentialRecord } = this.credentialMessageBuilder.createProposal(formats, proposal);
        credentialRecord.credentialAttributes = (_a = proposalMessage.credentialProposal) === null || _a === void 0 ? void 0 : _a.attributes;
        credentialRecord.connectionId = proposal.connectionId;
        this.logger.debug('Save meta data and emit state change event');
        await this.credentialRepository.save(credentialRecord);
        this.eventEmitter.emit({
            type: CredentialEvents_1.CredentialEventTypes.CredentialStateChanged,
            payload: {
                credentialRecord,
                previousState: null,
            },
        });
        await this.didCommMessageRepository.saveOrUpdateAgentMessage({
            agentMessage: proposalMessage,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        for (const format of formats) {
            const options = {
                credentialRecordId: credentialRecord.id,
                credentialFormats: {},
                protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V2,
            };
            options.proposalAttachment = format.getAttachment(proposalMessage.formats, proposalMessage.messageAttachment);
            await format.processProposal(options, credentialRecord);
        }
        return { credentialRecord, message: proposalMessage };
    }
    /**
     * Method called by {@link V2ProposeCredentialHandler} on reception of a propose credential message
     * We do the necessary processing here to accept the proposal and do the state change, emit event etc.
     * @param messageContext the inbound propose credential message
     * @returns credential record appropriate for this incoming message (once accepted)
     */
    async processProposal(messageContext) {
        let credentialRecord;
        const { message: proposalMessage, connection } = messageContext;
        this.logger.debug(`Processing credential proposal with id ${proposalMessage.id}`);
        try {
            // Credential record already exists
            credentialRecord = await this.getByThreadAndConnectionId(proposalMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
            // this may not be the first proposal message...
            // let proposalCredentialMessage, offerCredentialMessage
            // try {
            const proposalCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: V2ProposeCredentialMessage_1.V2ProposeCredentialMessage,
            });
            const offerCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: V2OfferCredentialMessage_1.V2OfferCredentialMessage,
            });
            credentialRecord.assertState(CredentialState_1.CredentialState.OfferSent);
            this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
                previousReceivedMessage: proposalCredentialMessage !== null && proposalCredentialMessage !== void 0 ? proposalCredentialMessage : undefined,
                previousSentMessage: offerCredentialMessage !== null && offerCredentialMessage !== void 0 ? offerCredentialMessage : undefined,
            });
            // Update record
            await this.didCommMessageRepository.saveOrUpdateAgentMessage({
                agentMessage: proposalMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
            await this.updateState(credentialRecord, CredentialState_1.CredentialState.ProposalReceived);
        }
        catch (_a) {
            // No credential record exists with thread id
            // get the format service objects for the formats found in the message
            credentialRecord = this.credentialMessageBuilder.processProposal(proposalMessage, connection === null || connection === void 0 ? void 0 : connection.id);
            // Save record and emit event
            this.connectionService.assertConnectionOrServiceDecorator(messageContext);
            await this.credentialRepository.save(credentialRecord);
            await this.didCommMessageRepository.saveOrUpdateAgentMessage({
                agentMessage: proposalMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
            await this.emitEvent(credentialRecord);
        }
        return credentialRecord;
    }
    async acceptProposal(proposal, credentialRecord) {
        var _a;
        const options = {
            connectionId: (_a = proposal.connectionId) !== null && _a !== void 0 ? _a : undefined,
            protocolVersion: proposal.protocolVersion,
            credentialFormats: proposal.credentialFormats,
            comment: proposal.comment,
        };
        const message = await this.createOfferAsResponse(credentialRecord, options);
        return { credentialRecord, message };
    }
    /**
     * Create a {@link AcceptProposalOptions} object used by handler
     *
     * @param credentialRecord {@link CredentialRecord} the record containing the proposal
     * @return options attributes of the proposal
     *
     */
    async createAcceptProposalOptions(credentialRecord) {
        const proposalMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2ProposeCredentialMessage_1.V2ProposeCredentialMessage,
        });
        if (!proposalMessage) {
            throw new error_1.AriesFrameworkError(`Missing proposal message for credential record ${credentialRecord.id}`);
        }
        const formats = this.getFormatsFromMessage(proposalMessage.formats);
        if (!formats || formats.length === 0) {
            throw new error_1.AriesFrameworkError(`Unable to create accept proposal options. No supported formats`);
        }
        const options = {
            credentialRecordId: credentialRecord.id,
            credentialFormats: {},
            protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V2,
        };
        for (const formatService of formats) {
            options.proposalAttachment = formatService.getAttachment(proposalMessage.formats, proposalMessage.messageAttachment);
            // should fill in the credential formats
            await formatService.processProposal(options, credentialRecord);
        }
        return options;
    }
    /**
     * Negotiate a credential proposal as issuer (by sending a credential offer message) to the connection
     * associated with the credential record.
     *
     * @param options configuration for the offer see {@link NegotiateProposalOptions}
     * @returns Credential exchange record associated with the credential offer
     *
     */
    async negotiateProposal(options, credentialRecord) {
        if (!credentialRecord.connectionId) {
            throw new error_1.AriesFrameworkError(`No connectionId found for credential record '${credentialRecord.id}'. Connection-less issuance does not support negotiation.`);
        }
        const message = await this.createOfferAsResponse(credentialRecord, options);
        return { credentialRecord, message };
    }
    /**
     * Create a {@link ProposePresentationMessage} as response to a received credential offer.
     * To create a proposal not bound to an existing credential exchange, use {@link createProposal}.
     *
     * @param credentialRecord The credential record for which to create the credential proposal
     * @param config Additional configuration to use for the proposal
     * @returns Object containing proposal message and associated credential record
     *
     */
    async negotiateOffer(options, credentialRecord) {
        var _a;
        // Assert
        credentialRecord.assertState(CredentialState_1.CredentialState.OfferReceived);
        // Create message
        const formats = this.getFormats(options.credentialFormats);
        if (!formats || formats.length === 0) {
            throw new error_1.AriesFrameworkError(`Unable to negotiate offer. No supported formats`);
        }
        const { message: credentialProposalMessage } = this.credentialMessageBuilder.createProposal(formats, options);
        credentialProposalMessage.setThread({ threadId: credentialRecord.threadId });
        // Update record
        await this.didCommMessageRepository.saveOrUpdateAgentMessage({
            agentMessage: credentialProposalMessage,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        credentialRecord.credentialAttributes = (_a = credentialProposalMessage.credentialProposal) === null || _a === void 0 ? void 0 : _a.attributes;
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.ProposalSent);
        return { message: credentialProposalMessage, credentialRecord };
    }
    /**
     * Create a {@link V2OfferCredentialMessage} as beginning of protocol process.
     *
     * @param formatService {@link CredentialFormatService} the format service object containing format-specific logic
     * @param options attributes of the original offer
     * @returns Object containing offer message and associated credential record
     *
     */
    async createOffer(options) {
        if (!options.connectionId) {
            throw new error_1.AriesFrameworkError('Connection id missing from offer credential options');
        }
        const connection = await this.connectionService.getById(options.connectionId);
        connection === null || connection === void 0 ? void 0 : connection.assertReady();
        const formats = this.getFormats(options.credentialFormats);
        if (!formats || formats.length === 0) {
            throw new error_1.AriesFrameworkError(`Unable to create offer. No supported formats`);
        }
        // Create message
        const { credentialRecord, message: credentialOfferMessage } = await this.credentialMessageBuilder.createOffer(formats, options);
        credentialRecord.connectionId = options.connectionId;
        await this.credentialRepository.save(credentialRecord);
        await this.emitEvent(credentialRecord);
        await this.didCommMessageRepository.saveOrUpdateAgentMessage({
            agentMessage: credentialOfferMessage,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        return { credentialRecord, message: credentialOfferMessage };
    }
    /**
     * Create an offer message for an out-of-band (connectionless) credential
     * @param credentialOptions the options (parameters) object for the offer
     * @returns the credential record and the offer message
     */
    async createOutOfBandOffer(credentialOptions) {
        const formats = this.getFormats(credentialOptions.credentialFormats);
        if (!formats || formats.length === 0) {
            throw new error_1.AriesFrameworkError(`Unable to create out of band offer. No supported formats`);
        }
        // Create message
        const { credentialRecord, message: offerCredentialMessage } = await this.credentialMessageBuilder.createOffer(formats, credentialOptions);
        // Create and set ~service decorator
        const routing = await this.mediationRecipientService.getRouting();
        offerCredentialMessage.service = new ServiceDecorator_1.ServiceDecorator({
            serviceEndpoint: routing.endpoints[0],
            recipientKeys: [routing.verkey],
            routingKeys: routing.routingKeys,
        });
        await this.credentialRepository.save(credentialRecord);
        await this.didCommMessageRepository.saveOrUpdateAgentMessage({
            agentMessage: offerCredentialMessage,
            role: storage_1.DidCommMessageRole.Receiver,
            associatedRecordId: credentialRecord.id,
        });
        await this.emitEvent(credentialRecord);
        return { credentialRecord, message: offerCredentialMessage };
    }
    /**
     * Create a {@link OfferCredentialMessage} as response to a received credential proposal.
     * To create an offer not bound to an existing credential exchange, use {@link V2CredentialService#createOffer}.
     *
     * @param credentialRecord The credential record for which to create the credential offer
     * @param credentialTemplate The credential template to use for the offer
     * @returns Object containing offer message and associated credential record
     *
     */
    async createOfferAsResponse(credentialRecord, proposal) {
        var _a;
        // Assert
        credentialRecord.assertState(CredentialState_1.CredentialState.ProposalReceived);
        let options;
        if (!proposal) {
            const acceptProposalOptions = await this.createAcceptProposalOptions(credentialRecord);
            options = {
                credentialFormats: acceptProposalOptions.credentialFormats,
                protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V2,
                credentialRecordId: acceptProposalOptions.connectionId,
                comment: acceptProposalOptions.comment,
            };
        }
        else {
            options = proposal;
        }
        const formats = this.getFormats(options.credentialFormats);
        if (!formats || formats.length === 0) {
            throw new error_1.AriesFrameworkError(`Unable to create offer as response. No supported formats`);
        }
        // Create the offer message
        this.logger.debug(`Get the Format Service and Create Offer Message for credential record ${credentialRecord.id}`);
        const proposeCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2ProposeCredentialMessage_1.V2ProposeCredentialMessage,
        });
        const credentialOfferMessage = await this.credentialMessageBuilder.createOfferAsResponse(formats, credentialRecord, options);
        credentialOfferMessage.credentialPreview = proposeCredentialMessage === null || proposeCredentialMessage === void 0 ? void 0 : proposeCredentialMessage.credentialProposal;
        credentialRecord.credentialAttributes = (_a = proposeCredentialMessage === null || proposeCredentialMessage === void 0 ? void 0 : proposeCredentialMessage.credentialProposal) === null || _a === void 0 ? void 0 : _a.attributes;
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.OfferSent);
        await this.didCommMessageRepository.saveOrUpdateAgentMessage({
            agentMessage: credentialOfferMessage,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        return credentialOfferMessage;
    }
    /**
     * Method called by {@link V2OfferCredentialHandler} on reception of a offer credential message
     * We do the necessary processing here to accept the offer and do the state change, emit event etc.
     * @param messageContext the inbound offer credential message
     * @returns credential record appropriate for this incoming message (once accepted)
     */
    async processOffer(messageContext) {
        var _a;
        let credentialRecord;
        const { message: credentialOfferMessage, connection } = messageContext;
        this.logger.debug(`Processing credential offer with id ${credentialOfferMessage.id}`);
        const formats = this.getFormatsFromMessage(credentialOfferMessage.formats);
        if (!formats || formats.length === 0) {
            throw new error_1.AriesFrameworkError(`Unable to create offer. No supported formats`);
        }
        try {
            // Credential record already exists
            credentialRecord = await this.getByThreadAndConnectionId(credentialOfferMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
            const proposeCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: V2ProposeCredentialMessage_1.V2ProposeCredentialMessage,
            });
            const offerCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: V2OfferCredentialMessage_1.V2OfferCredentialMessage,
            });
            credentialRecord.assertState(CredentialState_1.CredentialState.ProposalSent);
            this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
                previousReceivedMessage: offerCredentialMessage !== null && offerCredentialMessage !== void 0 ? offerCredentialMessage : undefined,
                previousSentMessage: proposeCredentialMessage !== null && proposeCredentialMessage !== void 0 ? proposeCredentialMessage : undefined,
            });
            for (const format of formats) {
                const attachment = format.getAttachment(credentialOfferMessage.formats, credentialOfferMessage.messageAttachment);
                if (!attachment) {
                    throw new error_1.AriesFrameworkError(`Missing offer attachment in credential offer message`);
                }
                await format.processOffer(attachment, credentialRecord);
            }
            await this.updateState(credentialRecord, CredentialState_1.CredentialState.OfferReceived);
            await this.didCommMessageRepository.saveOrUpdateAgentMessage({
                agentMessage: credentialOfferMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
        }
        catch (error) {
            // No credential record exists with thread id
            this.logger.debug('No credential record found for this offer - create a new one');
            credentialRecord = new repository_1.CredentialExchangeRecord({
                connectionId: connection === null || connection === void 0 ? void 0 : connection.id,
                threadId: credentialOfferMessage.id,
                credentialAttributes: (_a = credentialOfferMessage.credentialPreview) === null || _a === void 0 ? void 0 : _a.attributes,
                state: CredentialState_1.CredentialState.OfferReceived,
                protocolVersion: CredentialProtocolVersion_1.CredentialProtocolVersion.V2,
                credentials: [],
            });
            for (const format of formats) {
                const attachment = format.getAttachment(credentialOfferMessage.formats, credentialOfferMessage.messageAttachment);
                if (!attachment) {
                    throw new error_1.AriesFrameworkError(`Missing offer attachment in credential offer message`);
                }
                await format.processOffer(attachment, credentialRecord);
            }
            // Save in repository
            this.logger.debug('Saving credential record and emit offer-received event');
            await this.credentialRepository.save(credentialRecord);
            await this.didCommMessageRepository.saveOrUpdateAgentMessage({
                agentMessage: credentialOfferMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
            this.eventEmitter.emit({
                type: CredentialEvents_1.CredentialEventTypes.CredentialStateChanged,
                payload: {
                    credentialRecord,
                    previousState: null,
                },
            });
        }
        return credentialRecord;
    }
    /**
     * Create a {@link V2RequestCredentialMessage}
     *
     * @param credentialRecord The credential record for which to create the credential request
     * @param options request options for creating this request
     * @returns Object containing request message and associated credential record
     *
     */
    async createRequest(record, options, holderDid // temporary workaround
    ) {
        this.logger.debug('Get the Format Service and Create Request Message');
        record.assertState(CredentialState_1.CredentialState.OfferReceived);
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: record.id,
            messageClass: V2OfferCredentialMessage_1.V2OfferCredentialMessage,
        });
        if (!offerMessage) {
            throw new errors_1.CredentialProblemReportError(`Missing required base64 or json encoded attachment data for credential offer with thread id ${record.threadId}`, { problemCode: errors_1.CredentialProblemReportReason.IssuanceAbandoned });
        }
        const formats = this.getFormatsFromMessage(offerMessage.formats);
        if (!formats || formats.length == 0) {
            throw new error_1.AriesFrameworkError('No format keys found on the RequestCredentialOptions object');
        }
        const optionsForRequest = {
            formatServices: formats,
            record,
            requestOptions: options,
            offerMessage,
            holderDid,
        };
        const { message, credentialRecord } = await this.credentialMessageBuilder.createRequest(optionsForRequest);
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.RequestSent);
        return { message, credentialRecord };
    }
    /**
     * Process a received {@link RequestCredentialMessage}. This will not accept the credential request
     * or send a credential. It will only update the existing credential record with
     * the information from the credential request message. Use {@link createCredential}
     * after calling this method to create a credential.
     *
     * @param messageContext The message context containing a v2 credential request message
     * @returns credential record associated with the credential request message
     *
     */
    async processRequest(messageContext) {
        const { message: credentialRequestMessage, connection } = messageContext;
        const credentialRecord = await this.getByThreadAndConnectionId(credentialRequestMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        credentialRecord.connectionId = connection === null || connection === void 0 ? void 0 : connection.id;
        const proposalMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2ProposeCredentialMessage_1.V2ProposeCredentialMessage,
        });
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2OfferCredentialMessage_1.V2OfferCredentialMessage,
        });
        // Assert
        credentialRecord.assertState(CredentialState_1.CredentialState.OfferSent);
        this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
            previousReceivedMessage: proposalMessage !== null && proposalMessage !== void 0 ? proposalMessage : undefined,
            previousSentMessage: offerMessage !== null && offerMessage !== void 0 ? offerMessage : undefined,
        });
        this.logger.debug('Credential record found when processing credential request', credentialRecord);
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: credentialRequestMessage,
            role: storage_1.DidCommMessageRole.Receiver,
            associatedRecordId: credentialRecord.id,
        });
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.RequestReceived);
        return credentialRecord;
    }
    /**
     * Create a {@link IssueCredentialMessage} as response to a received credential request.
     *
     * @param credentialRecord The credential record for which to create the credential
     * @param options Additional configuration to use for the credential
     * @returns Object containing issue credential message and associated credential record
     *
     */
    async createCredential(record, options) {
        var _a;
        record.assertState(CredentialState_1.CredentialState.RequestReceived);
        const requestMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: record.id,
            messageClass: V2RequestCredentialMessage_1.V2RequestCredentialMessage,
        });
        if (!requestMessage) {
            throw new error_1.AriesFrameworkError(`Missing credential request for credential exchange with thread id ${record.threadId}`);
        }
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: record.id,
            messageClass: V2OfferCredentialMessage_1.V2OfferCredentialMessage,
        });
        if (!offerMessage) {
            throw new error_1.AriesFrameworkError('Missing Offer Message in create credential');
        }
        const credentialFormats = this.getFormatsFromMessage(requestMessage.formats);
        if (!credentialFormats || credentialFormats.length === 0) {
            throw new error_1.AriesFrameworkError(`Unable to create credential. No supported formats`);
        }
        const { message: issueCredentialMessage, credentialRecord } = await this.credentialMessageBuilder.createCredential(credentialFormats, record, options, requestMessage, offerMessage);
        issueCredentialMessage.setThread({
            threadId: credentialRecord.threadId,
        });
        issueCredentialMessage.setPleaseAck();
        credentialRecord.autoAcceptCredential = (_a = options === null || options === void 0 ? void 0 : options.autoAcceptCredential) !== null && _a !== void 0 ? _a : credentialRecord.autoAcceptCredential;
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.CredentialIssued);
        return { message: issueCredentialMessage, credentialRecord };
    }
    /**
     * Process a received {@link IssueCredentialMessage}. This will not accept the credential
     * or send a credential acknowledgement. It will only update the existing credential record with
     * the information from the issue credential message. Use {@link createAck}
     * after calling this method to create a credential acknowledgement.
     *
     * @param messageContext The message context containing an issue credential message
     *
     * @returns credential record associated with the issue credential message
     *
     */
    async processCredential(messageContext) {
        const { message: issueCredentialMessage, connection } = messageContext;
        this.logger.debug(`Processing credential with id ${issueCredentialMessage.id}`);
        const credentialRecord = await this.getByThreadAndConnectionId(issueCredentialMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        credentialRecord.connectionId = connection === null || connection === void 0 ? void 0 : connection.id;
        const requestMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2RequestCredentialMessage_1.V2RequestCredentialMessage,
        });
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2OfferCredentialMessage_1.V2OfferCredentialMessage,
        });
        // Assert
        credentialRecord.assertState(CredentialState_1.CredentialState.RequestSent);
        this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
            previousReceivedMessage: offerMessage !== null && offerMessage !== void 0 ? offerMessage : undefined,
            previousSentMessage: requestMessage !== null && requestMessage !== void 0 ? requestMessage : undefined,
        });
        const formatServices = this.getFormatsFromMessage(issueCredentialMessage.formats);
        for (const formatService of formatServices) {
            // get the revocation registry and pass it to the process (store) credential method
            const issueAttachment = formatService.getAttachment(issueCredentialMessage.formats, issueCredentialMessage.messageAttachment);
            if (!issueAttachment) {
                throw new error_1.AriesFrameworkError('Missing credential attachment in processCredential');
            }
            const options = {
                credentialAttachment: issueAttachment,
            };
            await formatService.processCredential(options, credentialRecord);
        }
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.CredentialReceived);
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: issueCredentialMessage,
            role: storage_1.DidCommMessageRole.Receiver,
            associatedRecordId: credentialRecord.id,
        });
        return credentialRecord;
    }
    /**
     * Create a {@link V2CredentialAckMessage} as response to a received credential.
     *
     * @param credentialRecord The credential record for which to create the credential acknowledgement
     * @returns Object containing credential acknowledgement message and associated credential record
     *
     */
    async createAck(credentialRecord) {
        credentialRecord.assertState(CredentialState_1.CredentialState.CredentialReceived);
        // Create message
        const ackMessage = new V2CredentialAckMessage_1.V2CredentialAckMessage({
            status: common_1.AckStatus.OK,
            threadId: credentialRecord.threadId,
        });
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.Done);
        return { message: ackMessage, credentialRecord };
    }
    /**
     * Process a received {@link CredentialAckMessage}.
     *
     * @param messageContext The message context containing a credential acknowledgement message
     * @returns credential record associated with the credential acknowledgement message
     *
     */
    async processAck(messageContext) {
        const { message: credentialAckMessage, connection } = messageContext;
        this.logger.debug(`Processing credential ack with id ${credentialAckMessage.id}`);
        const credentialRecord = await this.getByThreadAndConnectionId(credentialAckMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        credentialRecord.connectionId = connection === null || connection === void 0 ? void 0 : connection.id;
        const requestMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2RequestCredentialMessage_1.V2RequestCredentialMessage,
        });
        const credentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2IssueCredentialMessage_1.V2IssueCredentialMessage,
        });
        // Assert
        credentialRecord.assertState(CredentialState_1.CredentialState.CredentialIssued);
        this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
            previousReceivedMessage: requestMessage !== null && requestMessage !== void 0 ? requestMessage : undefined,
            previousSentMessage: credentialMessage !== null && credentialMessage !== void 0 ? credentialMessage : undefined,
        });
        // Update record
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.Done);
        return credentialRecord;
    }
    /**
     * Register the v2 handlers. These handlers supplement, ie are created in addition to, the existing
     * v1 handlers.
     */
    registerHandlers() {
        this.logger.debug('Registering V2 handlers');
        this.dispatcher.registerHandler(new V2ProposeCredentialHandler_1.V2ProposeCredentialHandler(this, this.agentConfig, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new V2OfferCredentialHandler_1.V2OfferCredentialHandler(this, this.agentConfig, this.mediationRecipientService, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new V2RequestCredentialHandler_1.V2RequestCredentialHandler(this, this.agentConfig, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new V2IssueCredentialHandler_1.V2IssueCredentialHandler(this, this.agentConfig, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new V2CredentialAckHandler_1.V2CredentialAckHandler(this));
        this.dispatcher.registerHandler(new V2CredentialProblemReportHandler_1.V2CredentialProblemReportHandler(this));
    }
    // AUTO ACCEPT METHODS
    async shouldAutoRespondToProposal(options) {
        if (this.agentConfig.autoAcceptCredentials === CredentialAutoAcceptType_1.AutoAcceptCredential.Never) {
            return false;
        }
        if (options.credentialRecord.autoAcceptCredential === CredentialAutoAcceptType_1.AutoAcceptCredential.Never) {
            return false;
        }
        const proposalMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: options.credentialRecord.id,
            messageClass: V2ProposeCredentialMessage_1.V2ProposeCredentialMessage,
        });
        if (!proposalMessage) {
            throw new error_1.AriesFrameworkError('Missing proposal message in V2ProposeCredentialHandler');
        }
        const formatServices = this.getFormatsFromMessage(proposalMessage.formats);
        let shouldAutoRespond = true;
        for (const formatService of formatServices) {
            const formatShouldAutoRespond = this.agentConfig.autoAcceptCredentials == CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
                formatService.shouldAutoRespondToProposal(options);
            shouldAutoRespond = shouldAutoRespond && formatShouldAutoRespond;
        }
        return shouldAutoRespond;
    }
    shouldAutoRespondToOffer(credentialRecord, offerMessage, proposeMessage) {
        var _a;
        if (this.agentConfig.autoAcceptCredentials === CredentialAutoAcceptType_1.AutoAcceptCredential.Never) {
            return false;
        }
        let offerValues;
        let shouldAutoRespond = true;
        const formatServices = this.getFormatsFromMessage(offerMessage.formats);
        for (const formatService of formatServices) {
            let proposalAttachment;
            if (proposeMessage) {
                proposalAttachment = formatService.getAttachment(proposeMessage.formats, proposeMessage.messageAttachment);
            }
            const offerAttachment = formatService.getAttachment(offerMessage.formats, offerMessage.messageAttachment);
            offerValues = (_a = offerMessage.credentialPreview) === null || _a === void 0 ? void 0 : _a.attributes;
            const handlerOptions = {
                credentialRecord,
                autoAcceptType: this.agentConfig.autoAcceptCredentials,
                messageAttributes: offerValues,
                proposalAttachment,
                offerAttachment,
            };
            const formatShouldAutoRespond = this.agentConfig.autoAcceptCredentials == CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
                formatService.shouldAutoRespondToProposal(handlerOptions);
            shouldAutoRespond = shouldAutoRespond && formatShouldAutoRespond;
        }
        return shouldAutoRespond;
    }
    shouldAutoRespondToRequest(credentialRecord, requestMessage, proposeMessage, offerMessage) {
        const formatServices = this.getFormatsFromMessage(requestMessage.formats);
        let shouldAutoRespond = true;
        for (const formatService of formatServices) {
            let proposalAttachment, offerAttachment, requestAttachment;
            if (proposeMessage) {
                proposalAttachment = formatService.getAttachment(proposeMessage.formats, proposeMessage.messageAttachment);
            }
            if (offerMessage) {
                offerAttachment = formatService.getAttachment(offerMessage.formats, offerMessage.messageAttachment);
            }
            if (requestMessage) {
                requestAttachment = formatService.getAttachment(requestMessage.formats, requestMessage.messageAttachment);
            }
            const handlerOptions = {
                credentialRecord,
                autoAcceptType: this.agentConfig.autoAcceptCredentials,
                proposalAttachment,
                offerAttachment,
                requestAttachment,
            };
            const formatShouldAutoRespond = this.agentConfig.autoAcceptCredentials == CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
                formatService.shouldAutoRespondToRequest(handlerOptions);
            shouldAutoRespond = shouldAutoRespond && formatShouldAutoRespond;
        }
        return shouldAutoRespond;
    }
    shouldAutoRespondToCredential(credentialRecord, credentialMessage) {
        // 1. Get all formats for this message
        const formatServices = this.getFormatsFromMessage(credentialMessage.formats);
        // 2. loop through found formats
        let shouldAutoRespond = true;
        let credentialAttachment;
        for (const formatService of formatServices) {
            if (credentialMessage) {
                credentialAttachment = formatService.getAttachment(credentialMessage.formats, credentialMessage.messageAttachment);
            }
            const handlerOptions = {
                credentialRecord,
                autoAcceptType: this.agentConfig.autoAcceptCredentials,
                credentialAttachment,
            };
            // 3. Call format.shouldRespondToProposal for each one
            const formatShouldAutoRespond = this.agentConfig.autoAcceptCredentials == CredentialAutoAcceptType_1.AutoAcceptCredential.Always ||
                formatService.shouldAutoRespondToCredential(handlerOptions);
            shouldAutoRespond = shouldAutoRespond && formatShouldAutoRespond;
        }
        return shouldAutoRespond;
    }
    async getOfferMessage(id) {
        return await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: id,
            messageClass: V2OfferCredentialMessage_1.V2OfferCredentialMessage,
        });
    }
    async getRequestMessage(id) {
        return await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: id,
            messageClass: V2RequestCredentialMessage_1.V2RequestCredentialMessage,
        });
    }
    async getCredentialMessage(id) {
        return await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: id,
            messageClass: V2IssueCredentialMessage_1.V2IssueCredentialMessage,
        });
    }
    update(credentialRecord) {
        return this.credentialRepository.update(credentialRecord);
    }
    /**
     * Returns the protocol version for this credential service
     * @returns v2 as this is the v2 service
     */
    getVersion() {
        return CredentialProtocolVersion_1.CredentialProtocolVersion.V2;
    }
    /**
     * Gets the correct formatting service for this credential record type, eg indy or jsonld. Others may be
     * added in the future.
     * Each formatting service knows how to format the message structure for the specific record type
     * @param credentialFormatType the format type, indy, jsonld, jwt etc.
     * @returns the formatting service.
     */
    getFormatService(credentialFormatType) {
        return this.serviceFormatMap[credentialFormatType];
    }
    async emitEvent(credentialRecord) {
        this.eventEmitter.emit({
            type: CredentialEvents_1.CredentialEventTypes.CredentialStateChanged,
            payload: {
                credentialRecord,
                previousState: null,
            },
        });
    }
    /**
     * Retrieve a credential record by connection id and thread id
     *
     * @param connectionId The connection id
     * @param threadId The thread id
     * @throws {RecordNotFoundError} If no record is found
     * @throws {RecordDuplicateError} If multiple records are found
     * @returns The credential record
     */
    getByThreadAndConnectionId(threadId, connectionId) {
        return this.credentialRepository.getSingleByQuery({
            connectionId,
            threadId,
        });
    }
    /**
     * Get all the format service objects for a given credential format from an incoming message
     * @param messageFormats the format objects containing the format name (eg indy)
     * @return the credential format service objects in an array - derived from format object keys
     */
    getFormatsFromMessage(messageFormats) {
        const formats = [];
        for (const msg of messageFormats) {
            if (msg.format.includes('indy')) {
                formats.push(this.getFormatService(CredentialsModuleOptions_1.CredentialFormatType.Indy));
            }
            else if (msg.format.includes('aries')) {
                // todo
            }
            else {
                throw new error_1.AriesFrameworkError(`Unknown Message Format: ${msg.format}`);
            }
        }
        return formats;
    }
    /**
     * Get all the format service objects for a given credential format
     * @param credentialFormats the format object containing various optional parameters
     * @return the credential format service objects in an array - derived from format object keys
     */
    getFormats(credentialFormats) {
        const formats = [];
        const formatKeys = Object.keys(credentialFormats);
        for (const key of formatKeys) {
            const credentialFormatType = CredentialFormatServiceOptions_1.FORMAT_KEYS[key];
            const formatService = this.getFormatService(credentialFormatType);
            formats.push(formatService);
        }
        return formats;
    }
};
V2CredentialService = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __metadata("design:paramtypes", [ConnectionService_1.ConnectionService,
        repository_1.CredentialRepository,
        EventEmitter_1.EventEmitter,
        Dispatcher_1.Dispatcher,
        AgentConfig_1.AgentConfig,
        routing_1.MediationRecipientService,
        storage_1.DidCommMessageRepository,
        IndyCredentialFormatService_1.IndyCredentialFormatService,
        services_1.RevocationService])
], V2CredentialService);
exports.V2CredentialService = V2CredentialService;
//# sourceMappingURL=V2CredentialService.js.map